<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use DateTime;

class Tco01100DocumentoProposta {
    protected $environment;
    protected $entityManager;
    protected $tco01100DocumentoPropostaService;

    /**
     * @var {Array[String]} allowedFileTypePattern Tipos
     * de arquivos que são aceitos no sistema.
     */
    private $allowedFileTypePattern = array(
        "application/octet-stream"=>array(),
        "image/png"=>array('png'),
        "image/bmp"=>array('bmp'),
        "image/jpeg"=>array('jpg', 'jpeg'),
        "text/plain"=>array('txt'),
        "image/x-png"=>array('png'),
        "application/rtf"=>array('rtf'),
        "application/pdf"=>array('pdf'),
        "application/msword"=>array('doc'),
        "image/x-citrix-png"=>array('png'),
        "application/wps-office.doc"=>array('doc'),
        "application/wps-office.docx"=>array('docx'),
        "application/wps-office.xls"=>array('xls'),
        "application/wps-office.xlsx"=>array('xlsx'),
        "application/wps-office.ppt"=>array('ppt'),
        "application/wps-office.pptx"=>array('pptx'),
        "image/x-citrix-jpeg"=>array('jpg', 'jpeg'),
        "application/vnd.ms-excel"=>array('xls', 'xlsx'),
        "application/vnd.openxmlformats-officedocument.wordprocessingml.document"=>array('pptx'),
        "application/vnd.ms-powerpoint"=>array('ppt', 'pptx'),
        "application/vnd.openxmlformats-officedocument.presentationml.presentation"=>array('docx'),
    );
    
    public function __construct($environment, $entityManager, $tco01100DocumentoPropostaService) {
        $this->environment                       = $environment;
        $this->entityManager                     = $entityManager;
        $this->tco01100DocumentoPropostaService  = $tco01100DocumentoPropostaService;
    }

    /**
     * Faz upload do arquivo para o fileServer do 
     * sistema.
     * 
     * @param {DTO\Request} $request Requisição vinda
     *  do usuário.
     * @param {DTO\Reponse} $response Instância de 
     *  resposta a requisição do usuário.
     */
    public function uploadFile($request, $response){
        $row = $request->getRow();
        $nrPropostaComercial = $row->get('NRPROPOSTACOMERCIAL');
        $isValidResponsavelPropCom = $this->isValidResponsavelPropCom($nrPropostaComercial);
        $isValidFileToUpload = $this->isValidFileToUpload($row);

        if($isValidResponsavelPropCom=='VALID'){

            if($isValidFileToUpload=='VALID'){
                $this->tco01100DocumentoPropostaService->uploadFile($request, $response);
            } else {
                $response->setError(new Error($isValidFileToUpload, 1));
            }

        } else {
            $response->setError(new Error($isValidResponsavelPropCom, 1));
        }
    }

    /**
     * Faz download do arquivo para o cliente.
     * 
     * @param {DTO\Request} $request Requisição vinda
     *  do usuário.
     * @param {DTO\Reponse} $response Instância de 
     *  resposta a requisição do usuário.
     */
    public function downloadFile($request, $response){
        $row = $request->getRow();
        $docNum = $this->tco01100DocumentoPropostaService->getDocPropComDetails(array($row->get('NRDOCUMENTOPROPCOM')))[0];
        $nrPropostaComercial = $docNum['NRPROPOSTACOMERCIAL'];
        
        if($this->isValidResponsavelPropCom($nrPropostaComercial)){
            $fileToDownload = $this->tco01100DocumentoPropostaService->downloadFile($row->get('NRDOCUMENTOPROPCOM'));
            
            if($fileToDownload){

                $fileToDownload['FILE_TYPE'] = $this->getFileTypeByName($fileToDownload['FILE_NAME']);
                $response->addDataSet(new DataSet('fileToDownload', $fileToDownload));

            } else {
                $response->setError(new Error('TCO_FILE_NOT_EXISTS', 1));
            }
        } else {
            $response->setError(new Error('TCO01100_INVALID_RESPONSAVEL_PROPCOM', 1));
        }
    }

    /**
     * Retorna o tipo do arquivo com base no nome
     * salvo no servidor.
     * 
     * @param {String} $fileName Nome do arquivo.
     * 
     * @return {String|Boolean} Tipo do arquivo caso
     *  seja tipado e falso caso não exista o tipo.
     */
    public function getFileTypeByName($fileName){
        foreach ($this->allowedFileTypePattern as $type => $ext) {
            $fileExt = explode('.', $fileName)[1];
            if(in_array($fileExt, $ext)){
                return $type;
            }
        }
        return false;
    }

    /**
     * Remove arquivo do fileServer do sistema e
     * o registro do banco de dados.
     * 
     * @param {DTO\Request} $request Requisição vinda
     *  do usuário.
     * @param {DTO\Reponse} $response Instância de 
     *  resposta a requisição do usuário.
     */
    public function removeFile($request, $response){
        $responseRow = $request->getRow();
        $nrPropostaComercial = $responseRow->get('NRPROPOSTACOMERCIAL');

        $rowsToRemove = array_map(function($row){
            return $row['NRDOCUMENTOPROPCOM'];
        }, $responseRow->get('FILES'));

        if($this->isValidResponsavelPropCom($nrPropostaComercial)){
            $this->tco01100DocumentoPropostaService->removeFile($rowsToRemove);
        } else {
            $response->setError(new Error('TCO01100_INVALID_RESPONSAVEL_PROPCOM', 1));
        }
    }

     /**
     * Valida arquivo selecionado na tela de
     * novo documento e exibe mensagem ao 
     * usuário caso seja inválido.
     * 
     * @param {DTO\Row} $row Registro contendo
     *  arquivo a ser inserido.
     */
    private function isValidFileToUpload($row){
        $b64File = explode(';',$row->get('FILE')[0]['b64File']);
        $returnState = 'TCO01100_INVALID_FILE_TYPE';

        if(sizeof($b64File)){
            $b64FileType = explode('data:',$b64File[0])[1];

            if(in_array($b64FileType, array_keys($this->allowedFileTypePattern))){
                $returnState = 'VALID';
            }
        }

        return $returnState;
    }

    /**
     * Retorna o estado de validade do operador 
     * logado como responsavel pela propostacomercial
     * atual.
     */
    private function isValidResponsavelPropCom($nrPropostaComercial){
        $cdOperador = $this->environment->getCdOperador();
        $isValid = $this->tco01100DocumentoPropostaService->isValidResponsavelPropCom($cdOperador, $nrPropostaComercial);

        if($isValid) return 'VALID';
        else return 'TCO01100_INVALID_RESPONSAVEL_PROPCOM';
    }
}